const hyprland = await Service.import("hyprland")
const notifications = await Service.import("notifications")
const mpris = await Service.import("mpris")
const audio = await Service.import("audio")
const battery = await Service.import("battery")
const systemtray = await Service.import("systemtray")
import { getIconName } from "./utils.js"

const volumeIndicator = Widget.Button({
    on_clicked: () => audio.speaker.is_muted = !audio.speaker.is_muted,
    class_name: "item",
    child: Widget.Box({
    children: [
      Widget.Icon().hook(audio.speaker, self => {
            const vol = audio.speaker.volume * 100
            const icon = [
                [101, 'overamplified'],
                [67, 'high'],
                [34, 'medium'],
                [1, 'low'],
                [0, 'muted'],
            ].find(([threshold]) => threshold <= vol)?.[1]

            self.icon = `audio-volume-${icon}-symbolic`
        self.tooltip_text = `Volume ${Math.floor(vol)}%`
      }),
      Widget.Label().hook(audio.speaker, self => {
        self.label = `${Math.floor(audio.speaker.volume * 100)}%`
        self.css = "margin-left: 0.7em;"
      })
    ]}),
});

function Clients() {
  const activeId = hyprland.active.client.bind("address")
  const clients = hyprland.bind("clients").as(cl =>
    cl.map(({ address, title, workspace }) =>
    {
        const short_title = title.length > 40 ? title.slice(0, 20) + "..." : title
        return Widget.Button({
          attribute: workspace.id,
          child: Widget.Box({
            children: [
              Widget.Icon({
                vexpand: false,
                size: 16,
                className: "app-icon",
                icon: getIconName(hyprland.clients.find(c => c.address === address))
              }),
              Widget.Label(`${short_title}`)
            ],
          }),
          class_name: activeId.as(i => `${i === address ? "item client-title focused" : "item client-title"}`),
        })
      }
    )
  )

  return Widget.Box({
    class_name: "clients",
    children: clients,
    setup: self => self.hook(hyprland, () => self.children.forEach(btn => {
      btn.visible = hyprland.active.workspace.id === btn.attribute
    })),
  })
}

function Workspaces() {
  const activeId = hyprland.active.workspace.bind("id")
  const workspaces = hyprland.bind("workspaces").as(ws => {
    ws.sort((a, b) => a.id - b.id)
    return ws.map(({ id, monitorID }) =>
      Widget.Button({
        attribute: monitorID,
        label: `${id}`.slice(-1),
        onClicked: () => hyprland.messageAsync(`dispatch workspace ${id}`),
        class_name: activeId.as(i => `${i === id ? "focused" : ""}`),
      })
    )
  })

  return Widget.Box({
    children: workspaces,
    class_name: "workspaces",
    setup: self => self.hook(hyprland, () => self.children.forEach(btn => {
      btn.visible = hyprland.active.monitor.id === btn.attribute
    })),
  })
}

function Notification() {
  const popups = notifications.bind("popups")
  return Widget.Box({
    class_name: "notification",
    visible: popups.as(p => p.length > 0),
    children: [
      Widget.Icon({
        icon: "preferences-system-notifications-symbolic",
      }),
      Widget.Label({
        label: popups.as(p => p[0]?.summary || ""),
      }),
    ],
  })
}

function SysTray() {
  const items = systemtray.bind("items")
  .as(items => items.map(item => Widget.Button({
    child: Widget.Icon({
      icon: item.bind("icon"),
      css: "margin-left: 0.6em;margin-right: 0.6em;"
    }),
    on_primary_click: (_, event) => item.activate(event),
    on_secondary_click: (_, event) => item.openMenu(event),
    tooltip_markup: item.bind("tooltip_markup"),
    css: "margin-left: 0.4em;",
  })))

  return Widget.Box({
    children: items,
  })
}

function Left() {
  return Widget.Box({
    spacing: 8,
    children: [
      Clients(),
    ],
  })
}

function Center() {
  return Widget.Box({
    spacing: 8,
    children: [
      Workspaces(),
      Notification(),
    ],
  })
}

function Right() {
  return Widget.Box({
    hpack: "end",
    spacing: 8,
    children: [
      SysTray(),
      volumeIndicator,
      Widget.Label({
        class_name: "item",
        label: Variable("", { poll: [1000, 'date "+%Y-%m-%d"'] }).bind(),
      }),
      Widget.Label({
        class_name: "item",
        label: Variable("", { poll: [1000, 'date "+%H:%M:%S"'] }).bind(),
      })
    ],
  })
}

function Bar(monitor = 0) {
  return Widget.Window({
    name: `bar-${monitor}`, // name has to be unique
    class_name: "bar",
    monitor,
    anchor: ["top", "left", "right"],
    exclusivity: "exclusive",
    child: Widget.CenterBox({
      class_name: "window-box",
      start_widget: Left(),
      center_widget: Center(),
      end_widget: Right(),
    }),
  })
}

App.config({
  style: "./style.css",
  windows: [
    Bar(1),
  ],
})

export { }
